use library_loader::load_function_ptrs_from_lib;

use anyhow::Result;

use std::ffi::CStr;
use std::os::raw::c_char;

load_function_ptrs_from_lib! (GameModeFunctions, "libgamemode.so", {
    real_gamemode_request_start() -> i32,
    real_gamemode_request_end() -> i32,
    real_gamemode_error_string() -> *const c_char,
});

pub struct GameMode {
    game_mode_functions: GameModeFunctions,
}

impl GameMode {
    pub fn new() -> Result<GameMode> {
        let game_mode_functions = GameModeFunctions::load()?;

        if unsafe { game_mode_functions.real_gamemode_request_start() } < 0 {
            return Err(anyhow::Error::msg(format!(
                "Gamemode Error: {}",
                Self::string(unsafe { game_mode_functions.real_gamemode_error_string() })
            )));
        }

        println!("GameMode enabled");

        Ok(GameMode {
            game_mode_functions,
        })
    }

    fn string(msg: *const c_char) -> String {
        let c_str: &CStr = unsafe { CStr::from_ptr(msg) };
        let str_slice: &str = c_str.to_str().unwrap();
        str_slice.to_string()
    }
}

impl Drop for GameMode {
    fn drop(&mut self) {
        if unsafe { self.game_mode_functions.real_gamemode_request_end() } < 0 {
            println!(
                "Error disabling GameMode: {}",
                Self::string(unsafe { self.game_mode_functions.real_gamemode_error_string() })
            );

            return;
        }

        println!("GameMode disabled");
    }
}
