use std::collections::HashMap;

use crate::prelude::*;

#[derive(Debug, Clone)]
pub struct EngineCreateInfo<'a> {
    pub app_info: ApplicationInfo,

    pub window_info: WindowCreateInfo,

    pub os_specific_config: OsSpecificConfig,

    pub vulkan_debug_info: VulkanDebugInfo,
    pub volume_info: HashMap<String, f32>,

    pub gui_info: GuiHandlerCreateInfo<'a>,

    pub enable_backtrace: bool,
    pub enable_mouse: bool,
    pub enable_keyboard: bool,
    pub enable_controller: bool,

    pub controller_deadzones: ControllerDeadzones,

    pub resource_base_path: String,

    // engine settings
    pub controller_directories: ControllerPictureDirectories,
    pub asset_directories: AssetDirectories,

    pub graphics_info: GraphicsInfo,
    pub raytracing_info: RaytracingInfo,
    pub rasterizer_info: RasterizerInfo,
    pub key_backgrounds: ButtonBackgrounds,
}

impl<'a> Default for EngineCreateInfo<'a> {
    fn default() -> Self {
        Self {
            app_info: ApplicationInfo {
                application_name: "Default Application Name".to_string(),
                application_version: 1,
                engine_name: "Default Engine Name".to_string(),
                engine_version: 1,
            },
            window_info: WindowCreateInfo {
                title: "Default Window Title".to_string(),
                width: 1600,
                height: 900,
                fullscreen: false,
                requested_display: None,
            },
            os_specific_config: OsSpecificConfig {
                enable_game_mode: true,
            },
            vulkan_debug_info: VulkanDebugInfo {
                debugging: false,
                steam_layer: false,
                verbose: false,
                renderdoc: false,
            },
            volume_info: HashMap::new(),
            gui_info: GuiHandlerCreateInfo::new(),
            enable_backtrace: true,
            enable_mouse: true,
            enable_keyboard: true,
            enable_controller: true,
            controller_deadzones: ControllerDeadzones::default(),
            resource_base_path: String::new(),
            controller_directories: ControllerPictureDirectories::default(),
            asset_directories: AssetDirectories::default(),
            graphics_info: GraphicsInfo {
                sample_count: VK_SAMPLE_COUNT_4_BIT,
                render_type: SceneType::Raytracer,
                render_scale: 1.0,
                vsync: true,
            },
            raytracing_info: RaytracingInfo {
                triangles_per_as: 8,
                accelerate_buffer_size: 1024,
                stack_size: 32,

                max_lights: 32,
                recursion_depth: 3,
                use_default_pipeline: true,
                max_samplers: 1024,
            },
            rasterizer_info: RasterizerInfo {
                shadow_image_size: 1024,
                enable_lighting: true,
                use_deferred: false,
                use_shadow_maps: false,
            },
            key_backgrounds: ButtonBackgrounds::default(),
        }
    }
}
