#version 450

// input parameter
layout (location = 0) in vec3 in_position;

#ifdef NORMALS
	layout (location = 1) in vec3 in_normal;
#endif

#ifdef NORMAL_UVS
	layout (location = 2) in vec4 in_tangent;
#endif

#ifdef COLOR_UVS
	layout (location = 3) in vec2 in_uvs;
#endif

#ifdef COLOR
    layout (location = 3) in vec4 in_color;
#endif

#ifdef BONES
	layout (location = 4) in ivec4 in_bone_ids;
	layout (location = 5) in vec4 in_bone_weights;
#endif

// out parameter
layout (location = 0) out vec3 out_world_pos;

#ifdef COLOR_UVS
    layout (location = 1) out vec2 out_uv;
#endif

#ifdef COLOR
    layout (location = 1) out vec4 out_color;
#endif

#ifdef NORMALS
    #ifdef NORMAL_UVS
        layout (location = 2) out vec3 out_tangent;
    #endif

    layout (location = 3) out vec3 out_normal;
#endif

// descriptors
layout (set = 1, binding = 0) uniform CameraProperties
{
    mat4 view;
    mat4 proj;
    mat4 vp;

    mat4 inv_view;
    mat4 inv_proj;
} globals;

layout (set = 0, binding = 0) uniform ModelTransform
{
	mat4 m;
} model;

#ifdef BONES
	layout (set = 0, binding = 3) uniform BoneData
	{
		mat4 mat[BONE_COUNT];
	} bones;
#endif

void main()
{
	mat4 model_matrix = model.m;

	#ifdef BONES
		mat4 animation_matrix = (bones.mat[in_bone_ids[0]]) * in_bone_weights[0] +
								(bones.mat[in_bone_ids[1]]) * in_bone_weights[1] +
								(bones.mat[in_bone_ids[2]]) * in_bone_weights[2] +
								(bones.mat[in_bone_ids[3]]) * in_bone_weights[3];

		model_matrix = model_matrix * animation_matrix;
	#endif

	vec4 model_pos = model_matrix * vec4(in_position, 1.0);

    gl_Position = globals.vp * model_pos;
    out_world_pos = model_pos.xyz / model_pos.w;

    #ifdef COLOR_UVS
        out_uv = in_uvs;
    #endif

    #ifdef COLOR
        out_color = in_color;
    #endif

    #ifdef NORMALS
        mat3 inv_model_matrix = transpose(inverse(mat3(model_matrix)));

        out_normal = inv_model_matrix * normalize(in_normal.xyz);

        #ifdef NORMAL_UVS
            out_tangent = inv_model_matrix * normalize(in_tangent.xyz);
        #endif
    #endif
}
